/*
 * Copyright 2011-2016 David Karnok
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ix;

import java.util.Iterator;

final class IxTakeUntil<T> extends IxSource<T, T> {

    final IxPredicate<? super T> predicate;

    IxTakeUntil(Iterable<T> source, IxPredicate<? super T> predicate) {
        super(source);
        this.predicate = predicate;
    }

    @Override
    public Iterator<T> iterator() {
        return new TakeUntilIterator<T>(source.iterator(), predicate);
    }

    static final class TakeUntilIterator<T> extends IxSourceIterator<T, T> {

        final IxPredicate<? super T> predicate;

        TakeUntilIterator(Iterator<T> it, IxPredicate<? super T> predicate) {
            super(it);
            this.predicate = predicate;
        }

        @Override
        protected boolean moveNext() {
            if (it.hasNext()) {
                T v = it.next();
                value = v;
                hasValue = true;
                if (predicate.test(v)) {
                    done = true;
                }
                return true;
            }
            done = true;
            return false;
        }

        @Override
        public void remove() {
            it.remove();
        }
    }

}
